#!/usr/bin/env python
"""
#############################################################################################

SnakeDb ver 1.00

  ------------------------------------------------------------------------
Author: Ming Hui Huang
Company: Script Foundry Inc.
Info URL: http://www.scriptfoundry.com/modules/snakedb/
Download URL: http://www.scriptfoundry.com/modules/dist/
Contact: ming@scriptfoundry.com
  ------------------------------------------------------------------------

Introduction:

The purpose of SnakeDb is to provide an easy database API. You don't have
to write SQL statements in your scripts to create databases or execute a
query. You can access the fields in the tables through a dictionary. The
API also allows you to execute complex SQL statements. SnakeDb is a wrapper
around other python database modules. SnakeDb ver 1.00 supports Gadfly and
MySQL databases. You can easily extend it to use other python database
modules.

System Requirements:

   * Python 2.0
   * MySQL database system ver 3.22.25 and up
   * Gadfly python module
   * MySQLdb python module

SnakeDb Classes:

Below are a brief description of every class in the SnakeDb module.
Normally, you should not have to modify any of these classes.

   * SnakeDb - This is the base class of all SnakeDb database classes.
   * SnakeDbTable - This is the base class for defining a database table.
     You will need to create one class per database table.
   * SnakeDbObj - This base class provides access to the database tables.
     You will need to create one class per database table.
   * SnakeDbQuery - You create an instance of this class when you want to
     find a subset of entries that match a certain criteria. You can also
     specify the page number you want to jump to and the size of each page.
     Let's say there were 100 entries in the table that matches your search
     criteria. If you only want entries 50 to 59, you set page number to 5
     and page size to 10.
   * GadflySnakeDb - This class inherits from SnakeDb classs. GadflySnakeDb
     provides the interface to a gadlfy database.
   * MySQLdbSnakeDb - This class inherits from SnakeDb classs.
     MySQLdbSnakeDb provides the interface to a MySQL database.

Special Features:

There are few special features of SnakeDb. If you want to have a database
table called "Person". You will need to define the following class:

class PersonTable(SnakeDbTable):
        tableName="person"

        def __init__(self):
                sqlFields={
                #sqlfield: fieldtype
                'personId':'varchar(100)',
                'personObjId':'varchar(100)',
                'creationTimeStamp':'timestamp(14)',
                'firstName':'varchar(100)',
                'lastName':'varchar(100)',
                'phone':'varchar(100)',
                SnakeDbTable.__init__(self, self.tableName, sqlFields)

   * Notice the field "personId". This is a mandatory field which contains
     an unique id. At the creation of a new entry into the table, the
     unique id is generated from time.time(). So, the id might be something
     like "975423244.793". If you want to have a database table called
     "person", there must be a field called "personId".
   * The "personObjId" field is the encrypted version of the "personId".
     The "personObjId" field is optional. So, if you have a database table
     called "person" and there is a field "contactObjId", then the
     encrypted value of "personId" stored in the "personObjId".
   * The "creationTimeStamp" field will be automatically updated with a
     timestamp whenever the entry is modified with a "UPDATE" SQL
     statement. All fields ending with the string "TimeStamp" will be
     updated. The SnakeDb module actually puts the timestamp field in the
     following format: YYYYMMDDHHNNSS where YYYY is the year, MM is the
     month, DD is the day, HH is the hour, NN is the minute, and SS is the
     seconds. So if the table has the fields "field1timestamp" or
     "field2Timestamp" and the table entry is updated, then the fields will
     be updated.
   * You can access the table fields through a dictionary. First define the
     following Person class that inherits from SnakeDbObj.

     class Person(SnakeDbObj):
             def __init__(self, db, detailsDict={}):
                     SnakeDbObj.__init__(self, db, PersonTable.tableName, detailsDict)


     For example, if xVar is an instance of Person, then you can use
     xVar.details to access the fields. xVar.details['firstName'] gets you
     the firstName.
   * SnakeDb module also provides automatic paging for search query. If you
     want to go to the second page for all entries with the last name of
     "Lee", do the following:

     db=PhoneBookDb()
     db.connectDb()
     x=Person(db)
     q=SnakeDbQuery(snakeDb=db,pageSize=5,pageNumber=2)
     q.setCriteria({'lastName':'Lee'})
     results=x.find(q)

Installation:

  1. Download the distribution file at
     http://www.scriptfoundry.com/modules/dist/
  2. Uncompress the distribution file
  3. change into the directory created during the uncompression of
     distribution file
  4. execute the command:
     python setup.py install
     NOTE: Make sure you have the correct permissions before you run step
     4. If the python interpreter was installed by ROOT, a regular user
     probably run into trouble.

Unix Installation Example:

   * cd ~home
   * tar xvfz snakedb-1.00.tar.gz
   * cd snakedb-1.00
   * python setup.py install

Code Example:

Below is a script that shows how to create and use a Gadfly database.

#!/usr/bin/env python

from snakedb import *
import sys
from string import lower, join, split

class PhoneBookDb(GadflySnakeDb):
        dbDir="/tmp"
        dbName="phonebook"

        def __init__(self):
                GadflySnakeDb.__init__(self,self.dbName, self.dbDir)

        def registerAllDbTables(self):
                self.registerDbTable(PersonTable())
                self.registerDbTable(AddressTable())

class PersonTable(SnakeDbTable):
        tableName="person"

        def __init__(self):
                sqlFields={
                #sqlfield: fieldtype
                'personId':'varchar',
                'personObjId':'varchar',
                'updatedOnTimeStamp':'varchar',
                'lastName':'varchar',
                'firstName':'varchar',
                'phone':'varchar'
                }
                SnakeDbTable.__init__(self, self.tableName, sqlFields)

class Person(SnakeDbObj):
        def __init__(self, db, detailsDict={}):
                SnakeDbObj.__init__(self, db, PersonTable.tableName, detailsDict)


class AddressTable(SnakeDbTable):
        tableName="address"

        def __init__(self):
                sqlFields={
                #sqlfield: fieldtype
                'addressId':'varchar',
                'addressObjId':'varchar',
                'updatedOnTimeStamp':'varchar',
                'personId':'varchar',
                'city':'varchar',
                'state':'varchar'
                }
                SnakeDbTable.__init__(self, self.tableName, sqlFields)

class Address(SnakeDbObj):
        def __init__(self, db, detailsDict={}):
                SnakeDbObj.__init__(self, db, AddressTable.tableName, detailsDict)


db=PhoneBookDb()
try:
        db.dropDb()
except:
        pass
db.createDb()
print "New Phone Book database created!"
personInfo={
'firstName':'Ming',
'lastName':'Huang',
'phone':'212-228-0933'
}
p=Person(db, personInfo)
p.create()

addressInfo={
'personId':p.details['personId'],
'city':'Brooklyn',
'state': 'NY'
}
a=Address(db, addressInfo)
a.create()
print "Added new entry: %s %s"%(p.details,a.details)

print "Updating the address of Ming Huang"
a2=Address(db)
a2.load(a.details['addressId'])
a2.details['city']='Brooklyn'
a2.save()

personInfo={
'firstName':'Thanos',
'lastName':'Vassilakis',
'phone':'212-228-0933'
}
p=Person(db, personInfo)
p.create()

addressInfo={
'personId':p.details['personId'],
'city':'New York',
'state': 'NY'
}
a=Address(db, addressInfo)
a.create()

print "Added new entry: %s %s"%(p.details,a.details)

personInfo={
'firstName':'John',
'lastName':'Lee',
'phone':'212-555-5555'
}
p=Person(db, personInfo)
p.create()

addressInfo={

'personId':p.details['personId'],
'city':'Queens',
'state': 'NY'
}
a=Address(db, addressInfo)
a.create()

print "Added new entry: %s %s"%(p.details,a.details)

personInfo={
'firstName':'Sue',
'lastName':'Lee',
'phone':'212-555-6666'
}
p=Person(db, personInfo)
p.create()

addressInfo={
'personId':p.details['personId'],
'city':'Queens',
'state': 'NY'
}
a=Address(db, addressInfo)
a.create()

print "Added new entry: %s %s"%(p.details,a.details)

print "\n\nListing of all person with telephone: 212-228-0933"
query=SnakeDbQuery(snakeDb=db,pageSize=5,pageNumber=1)
query.setCriteria({'phone':'212-228-0933'})
print p.find(query)

print "\n\nListing of all person with telephone: 212-228-0933 and last name 'Huang'"
query=SnakeDbQuery(snakeDb=db,pageSize=5,pageNumber=1)
query.setCriteria({'phone':'212-228-0933', 'lastName':'Huang'})
print p.find(query)

print "\n\nDeleting all person who live in Brooklyn"
#Testing the deletion of entries in two tables
pgNumber=1
query=SnakeDbQuery(snakeDb=db,pageSize=5,pageNumber=pgNumber)
query.setCriteria({'city':'Brooklyn'})
a3=Address(db)
p3=Person(db)
found=a3.find(query)
while (found):
        for record in found:
                a3.remove(addressId=record['addressId'])
                p3.remove(personId=record['personId'])
        pgNumber=pgNumber+1
        query.reset(pageSize=5,pageNumber=pgNumber)
        found=a3.find(query)

print "\n\nDeleting all person with last name Lee"
#Testing the deletion of entries in only one table
#NOTE: In the following exmaple, the address entries for the deleted persons are not affected
p4=Person(db)
p4.remove(lastName='Lee')

db.closeDb()
#############################################################################################
"""

try:
	import MySQLdb
except:
	pass
try:
	import gadfly
except:
	pass
import time
import re
import types

def raiseDbExcept(*errStr):
	dbAPIexcept = 'SnakeDb Exception'
	raise dbAPIexcept, errStr

#This function generates the objId.
def hexDigestStr(str):
	import sha
	s=sha.sha()
	s.update(str)
	digestStr=s.hexdigest()
	return (digestStr[:10])

class SnakeDb:
	dbValidStates = ['NOTCONNECTED','CONNECTED','CLOSED']
	NOTCONNECTED=dbValidStates[0]
	CONNECTED=dbValidStates[1]
	CLOSED=dbValidStates[2]

	def __init__(self, dbName):
		self.dbName = dbName
		self.dbTables={}
		self.registerAllDbTables()
		self.setDbState(self.NOTCONNECTED)

	def sqlEscapeStr(self, mystr):
		raiseDbExcept("Undefined sqlEscapeStr()")

	def sqlUnescapeStr(self, mystr):
		raiseDbExcept("Undefined sqlUnescapeStr()")

	def createNewDb(self, dbName):
		raiseDbExcept("Undefined createNewDb()")

	def createAllDbTables(self):
		for tableName in self.dbTables.keys():
			self.dbTables[tableName].create()

	def createNewDbTables(self):
		for tableName in self.dbTables.keys():
			try:
				self.dbTables[tableName].create()
			except:
				pass

	def connectExistingDb(self, dbName):
		raiseDbExcept("Undefined connectExistingDb()")

	def executeSQL(self, queryObj, sql):
		cursor=queryObj.getCursor()
		cursor.execute(sql)
		if queryObj.getPageSize() == 0:
			return None
		gotoPage=queryObj.getPageNumber()
		for page in xrange(1,gotoPage+1):
			results=cursor.fetchmany(size=cursor.arraysize)
			if results==None:
				break
		try:
			#Make sure the "results" are a list of tuples.
			results=map(tuple, results)
		except:
			results=None	
		self.commitConnection()
		return results

	def closeDb(self):
		raiseDbExcept("Undefined closeDb()")
	
	def getDbNullStr(self):
		raiseDbExcept("Undefined getDbNullStr()")

	def getDbTimeStampStr(self):
		import time
		curtime=time.time()
		timestamp=time.strftime("%Y%m%d%H%M%S", time.localtime(curtime))
		return timestamp

	def getDbName(self):
		return self.dbName

	def getDbTable(self, tableName):
		return self.dbTables[tableName]

	def setDbState(self, state):
		if (state in self.dbValidStates):
			self.dbState=state
		else:
			raiseDbExcept("Undefined state in setDbState()", state)

	def getDbState(self):
		return self.dbState

	def getNextDbOid(self):
		id="%020.3f"%(time.time())
		return id

	def createDb(self):	
		self.createNewDb(self.dbName)
		self.createAllDbTables()

	def connectDb(self):	
		self.connectExistingDb(self.dbName)

	def dropDb(self):	
		self.dropExistingDb(self.dbName)

	def registerDbTable(self, dbTable):
		if not isinstance(dbTable, SnakeDbTable):
			raiseDbExcept("dbTable is not instance of SnakeDbTable")
		self.dbTables[dbTable.tableName]=dbTable
		dbTable.db=self

	def registerAllDbTables(self):
		pass

	def commitConnection(self):
		pass
	
class SnakeDbTable:

	def __init__(self, tableName, sqlFields):
		self.tableName=tableName
		#The db is set to None because when the table instance is created,
		#it should be registered into the database: self.db.registerDbTable(..)
		self.db=None
		#set the id field.
		#For example, the "person" table always has a "personId" field.
		idField="%sId"%tableName
		if not sqlFields.has_key(idField):
			raiseDbExcept("DbTable '%s' must have field: %s" %(tableName, idField))
		self.idField=idField
		self.objIdField="%sObjId"%tableName
		self.sqlFields=sqlFields
		self.sqlfieldnames=self.sqlFields.keys()
		self.selectOrder = ''

	def setSelectOrder(self, order):
		self.selectOrder = order

	def getSelectOrder(self):
		return self.selectOrder


	def create(self):
		sql=self.getCreateSQL()
		#Ming debug
		#print "dbName=", self.db.dbName
		#print "==createsql ",sql
		queryObj=SnakeDbQuery(snakeDb=self.db,pageSize=0,pageNumber=0)
		self.db.executeSQL(queryObj, sql)


	def insert(self, queryObj):
		#This list holds the id and objId.
		#For example, the list has personId and personObjId
		uniqueDbIds=[]
		sqlvalues=queryObj.getCriteria()
		if sqlvalues=={}:
			raiseDbExcept("no sqlvalues passed to SnakeDbTable.insert()")
		#If there is already an "dbId" value, don't overwrite it
		if (not sqlvalues.has_key(self.idField)) or (not sqlvalues[self.idField]):
			id = self.db.getNextDbOid()
			sqlvalues[self.idField]= id
		else:
			id=sqlvalues[self.idField]
		uniqueDbIds.append(id)	
		#If there is already an "dbObjId" value, don't overwrite it
		#Note: don't combine the if statements below.
		#The first "if" check for the objIdField and
		#the inner "if" check if objIdField value was passed to insert()
		if self.objIdField in self.sqlfieldnames:
			if (not sqlvalues.has_key(self.objIdField)) or (not sqlvalues[self.objIdField]):
				objId=hexDigestStr(id)
				sqlvalues[self.objIdField]=objId
			else:
				objId=sqlvalues[self.objIdField]
			uniqueDbIds.append(objId)	

		#Automatically timestamp all fields that end in "timestamp"
		#ie: createdTimestamp, updatedTimeStamp, deletetimestamp
		#Note: The match for the field name is case insensitive.
		timestamp=self.db.getDbTimeStampStr()
		import re
		pat=re.compile(".*timestamp$", re.IGNORECASE)
		for field in self.sqlfieldnames:
			if pat.match(field):	
				sqlvalues[field]=timestamp
		sql=self.getInsertSQL(sqlvalues)
		#Ming debug
		#print "\n<font color=green>==insertsql: %s</font>"%sql
		self.db.executeSQL(queryObj,sql)
		return uniqueDbIds

	def select(self, queryObj):
		sqlvalues=queryObj.getCriteria()
		sql=self.getSelectSQL(sqlvalues)
		if sqlvalues=={}:
			sqlvalues=self.sqlFields
		#Ming debug
		#print "\n<font color=blue>==selectsql: %s</font>"%sql
		results=self.db.executeSQL(queryObj, sql)
		#Unescape the values in results
		alldata=[]
		for dbrecord in results:
			data={}
			x=0
			for field in self.sqlfieldnames:
				if self.sqlFields.has_key(field):
					"""
					#Fix the 'None' bug
					if dbrecord[x]==None:
						data[field]=""
					else:
						data[field]=self.db.sqlUnescapeStr(dbrecord[x])
					"""
					#Ming 4-12-2000
					#If the db value is Null, data[field] will be None
					data[field]=self.db.sqlUnescapeStr(dbrecord[x])
					x=x+1
			alldata.append(data)
		#if alldata==[]:
		#	return None
		return alldata

	def delete(self, queryObj):
		sqlvalues=queryObj.getCriteria()
		if sqlvalues=={}:
			raiseDbExcept("no sqlvalues passed to SnakeDbTable.delete()")
		sql=self.getDeleteSQL(sqlvalues)
		self.db.executeSQL(queryObj,sql)

	def update(self, id, queryObj):
		sqlvalues=queryObj.getCriteria()
		if sqlvalues=={}:
			raiseDbExcept("no sqlvalues passed to SnakeDbTable.update()")
		sqlvalues[self.idField]=id
		#Automatically timestamp all fields that end in "timestamp"
		#ie: createdTimestamp, updatedTimeStamp, deletetimestamp
		#Note: The match for the field name is case insensitive.
		timestamp=self.db.getDbTimeStampStr()
		import re
		pat=re.compile(".*timestamp$", re.IGNORECASE)
		for field in self.sqlfieldnames:
			if pat.match(field):	
				sqlvalues[field]=timestamp
		sql=self.getUpdateSQL(sqlvalues)
		#Ming debug
		#print "\n<font color=orange>==updatesql: %s</font>"%sql
		self.db.executeSQL(queryObj, sql)
		return id

	def setSQLfields(self, osqlvalues):
		sqlvalues={}
		sqlvalues.update(osqlvalues)
		for field in self.sqlfieldnames:
			if (not sqlvalues.has_key(field)) or (sqlvalues[field]==None):
				sqlvalues[field]=None
			#Escape all special sql characters
			#Each database has its own way of escaping the characters.
			#if sqlvalues[field] and not type(sqlvalues[field]) is types.DictType:
			if type(sqlvalues[field]) == type("stringtype"): 
				sqlvalues[field]=self.db.sqlEscapeStr(str(sqlvalues[field]))
			elif not sqlvalues[field] and sqlvalues[field] !=0:
				sqlvalues[field]=None

		return sqlvalues

	def getInsertSQL(self, sqlvalues):
		sqlvalues=self.setSQLfields(sqlvalues)
		fieldstr=""
		valuestr=""
		for field in self.sqlfieldnames:
			if not field in sqlvalues.keys():
				continue
			if fieldstr!="":
				fieldstr=fieldstr+", "
				valuestr=valuestr+", "
			fieldstr=fieldstr+" %s" %field
			#Fix the "None" bug.
			if sqlvalues[field]==None:
				valuestr=valuestr+" %s" %self.db.getDbNullStr()
			else:
				valuestr=valuestr+" '%s'" %sqlvalues[field]
		sql="insert into %s (%s) values (%s)" % (self.tableName, fieldstr, valuestr)
		return sql
		
	def getUpdateSQL(self, sqlvalues):
		#print "<br>before<i>%s</i>"%sqlvalues
		sqlvalues=self.setSQLfields(sqlvalues)
		#print "<br>after<i>%s</i>"%sqlvalues
		fieldstr=""
		for field in self.sqlfieldnames:
			if not sqlvalues.has_key(field):
				continue
			#04-11-2000
			#Even though the programmer might assign a "None" value
			#to the field, this library stops the assignment of
			#Null value to the id and objId field. 
			if sqlvalues[field]==None and ((field==self.idField) or (field==self.objIdField)):
				continue
			if fieldstr!="":
				fieldstr=fieldstr+", "
			if sqlvalues[field]==None:
				fieldstr=fieldstr+" %s=%s" %(field, self.db.getDbNullStr())
			else:
				fieldstr=fieldstr+" %s='%s'" %(field, sqlvalues[field])
		sql="update %s set %s where %s='%s'" % (self.tableName, fieldstr, self.idField, sqlvalues[self.idField])
		return sql
		
	def getSelectSQL(self, sqlvalues):
		order = self.getSelectOrder()
		if order and (type(order)==type("string")):
			orderstmt = ' ORDER BY %s' % order
			#Reset the "select order"
			self.setSelectOrder('')
		else:
			orderstmt = ''
		if sqlvalues=={} or sqlvalues==None:
			fieldstr=""
			for field in self.sqlFields.keys():
				if fieldstr!="":
					fieldstr=fieldstr+", "
				fieldstr=fieldstr+" %s" %field
			sql="select %s from %s%s" % (fieldstr, self.tableName, orderstmt)
			return sql	
		else:
			sqlvalues=self.setSQLfields(sqlvalues)
		fieldstr=""
		wherestr=""
		for field in self.sqlfieldnames:
			if not field in sqlvalues.keys():
				continue
			if fieldstr!="":
				fieldstr=fieldstr+", "
			fieldstr=fieldstr+" %s" %field
			if sqlvalues[field]!=None:
				if wherestr!="":
					wherestr=wherestr+" and "
				wherestr= wherestr+" %s='%s'" %(field, sqlvalues[field])
		if wherestr!="":
			sql="select %s from %s where %s" %(fieldstr, self.tableName, wherestr)
		else:
			sql="select %s from %s" % (fieldstr, self.tableName)
		return sql
		
	def getDeleteSQL(self, sqlvalues):
		sqlvalues=self.setSQLfields(sqlvalues)
		wherestr=""
		for field in self.sqlfieldnames:
			if not field in sqlvalues.keys():
				continue
			if sqlvalues[field]!=None:
				if wherestr!="":
					wherestr=wherestr+" and "
				wherestr=wherestr+" %s='%s'" %(field, sqlvalues[field])
		if wherestr!="":
			sql="delete from %s where %s" %(self.tableName, wherestr)
		else:
			sql="delete from %s" % (self.tableName)
		return sql
		
	def getCreateSQL(self):
		fieldstr=""
		for field in self.sqlfieldnames:
			if fieldstr!="":
				fieldstr=fieldstr+", "
			fieldstr=fieldstr+"%s %s" %(field, self.sqlFields[field])
		sql="create table %s(%s)" % (self.tableName, fieldstr)
		return sql
		
		
class SnakeDbObj:

	def __init__(self, db, tableName, detailsDict=None, **detailsParams):
		#It is optional to supply details to the constructor.
		self.details={}
		self.details.update(detailsDict)
		if self.details==None:
			self.details.update(detailsParams)
		if not isinstance(db, SnakeDb):
			raiseDbExcept("db is not instance of SnakeDb")
		self.db=db
		self.tableName=tableName
		self.dbTable=db.getDbTable(self.tableName)
		self.dbObjId=None
		self.dbId=None

	def load(self, id):
		sqlparam={self.dbTable.idField:id}
		return self.loadByCriteria(sqlparam)

	def loadByObjId(self, id):
		if not self.dbTable.objIdField in self.dbTable.sqlfieldnames:
			return None
		sqlparam={self.dbTable.objIdField:id}
		return self.loadByCriteria(sqlparam)

	def loadByCriteria(self, detailsDict=None, **detailsParams):
		details=detailsDict
		if details==None:
			if detailsParams=={}:
				raiseDbExcept("missing parameters in loadByCriteria method")
			else:
				details=detailsParams
		if details=={}:
			raiseDbExcept("missing parameters in loadByCriteria method")
		queryObj=SnakeDbQuery(snakeDb=self.db,pageSize=1,pageNumber=1)
		queryObj.setCriteria(details)
		results=self.dbTable.select(queryObj)
		if results:
			self.details=results[0]
			self.dbId=results[0][self.dbTable.idField]
			self.dbObjId=id
			return self.details
		else:
			self.details=None
			return None	


	def create(self):
		queryObj=SnakeDbQuery(snakeDb=self.db,pageSize=0,pageNumber=0)
		queryObj.setCriteria(self.details)
		uniqueDbIds=self.dbTable.insert(queryObj)
		self.dbId=uniqueDbIds[0]
		self.details[self.dbTable.idField]=uniqueDbIds[0]
		if self.dbTable.objIdField in self.dbTable.sqlfieldnames:
			self.dbObjId=uniqueDbIds[1]
			self.details[self.dbTable.objIdField]=uniqueDbIds[1]
		return uniqueDbIds[0]

	def save(self):
		queryObj=SnakeDbQuery(snakeDb=self.db,pageSize=0,pageNumber=0)
		queryObj.setCriteria(self.details)
		self.dbTable.update(self.dbId, queryObj)

	def remove(self, detailsDict=None, **detailsParams):
		details=detailsDict
		if details==None:
			if detailsParams=={}:
				raiseDbExcept("missing parameters in remove method")
			else:
				details=detailsParams
		if details=={}:
			raiseDbExcept("missing parameters in remove method")
		queryObj=SnakeDbQuery(snakeDb=self.db,pageSize=0,pageNumber=0)
		queryObj.setCriteria(details)
		self.dbTable.delete(queryObj)

	def removeAll(self):
		queryObj=SnakeDbQuery(snakeDb=self.db,pageSize=0,pageNumber=0)
		self.dbTable.delete(queryObj)

	def find(self, queryObj):
		return self.dbTable.select(queryObj)
		

	def setDetails(self, detailsDict=None, **detailsParams):
		details=detailsDict
		if details==None:
			details=detailsParams
		self.details.update(details)

	def getDetails(self):
		return self.details

	def getDbObjId(self):
		return self.dbObjId

	def getDbId(self):
		return self.dbId

class SnakeDbQuery:

	def __init__(self, **kwparams):
		self.db=kwparams['snakeDb']
		self.__cursor=None
		self.reset(pageNumber=kwparams['pageNumber'],pageSize=kwparams['pageSize'])

	def getPageSize(self):
		return self.__pageSize

	def getPageNumber(self):
		return self.__curPage

	def reset(self, **kwparams):
		if self.__cursor:
			self.db.closeCursor(self.__cursor)
		self.__cursor=self.db.openCursor(kwparams['pageSize'])
		self.__criteria={}
		self.__curPage=kwparams['pageNumber']
		self.__pageSize=kwparams['pageSize']

	def setCriteria(self, dict):
		self.__criteria=dict

	def getCriteria(self):
		return self.__criteria

	def updateCriteria(self, dict):
		self.__criteria.update(dict)

	def clearCriteria(self):
		self.__criteria={}

	def getCursor(self):
		return self.__cursor

class GadflySnakeDb(SnakeDb):

	def __init__(self, dbName, dbDir):
		SnakeDb.__init__(self,dbName)
		self.__dbDir=dbDir
		self.__connection=None

	def getDbNullStr(self):
		return "''"

	def getDbDir(self):
		return self.__dbDir

	def sqlEscapeStr(self, mystr):
		if type(mystr)!=type("string"):
			return mystr
		mystr=re.sub(r"""(?P<sqlchar>['])""",r"""'\g<sqlchar>""", mystr)	
		mystr=re.sub(r"\r",r"\\r", mystr)	
		mystr=re.sub(r"\n",r"\\n", mystr)	
		return mystr

	def sqlUnescapeStr(self, mystr):
		if type(mystr)!=type("string"):
			return mystr
		mystr=re.sub(r"""'(?P<sqlchar>['])""",r"""\g<sqlchar>""", mystr)	
		mystr=re.sub(r"\\r",r"\r", mystr)	
		mystr=re.sub(r"\\n",r"\n", mystr)	
		return mystr

	def createNewDb(self, dbName):
		if self.getDbState() == self.CONNECTED:
			raiseDbExcept("Already connected to db")
		connection=gadfly.gadfly()
		connection.startup(dbName, self.dbDir)
		self.dbName=dbName
		self.__connection=connection
		connection.commit()
		self.setDbState(self.CONNECTED)

	def connectExistingDb(self, dbName):
		if self.getDbState() == self.CONNECTED:
			raiseDbExcept("Already connected to db")
		self.dbName=dbName
		connection=gadfly.gadfly(self.dbName, self.dbDir)
		self.__connection=connection
		self.setDbState(self.CONNECTED)


	def closeDb(self):
		self.__connection.commit()

	def openCursor(self, arraysize=10):
		cursor=self.__connection.cursor()
		#The arraysize determines how many results are fetched in cursor.execute()
		cursor.arraysize=arraysize
		return cursor

	def closeCursor(self, cursor):
		return cursor.close()

	def commitConnection(self):
		self.__connection.commit()

	def closeDb(self):
		self.commitConnection()
		self.__connection.close()

class MySQLdbSnakeDb(SnakeDb):

	def __init__(self, dbName, userHost, userName, userPassword):
		SnakeDb.__init__(self,dbName)
		self.__dbName=dbName
		self.__dbUserHost=userHost
		self.__dbUserName=userName
		self.__dbUserPassword=userPassword
		self.__connection=None

	def getDbNullStr(self):
		return "Null"

	def getDbUserHost(self):
		return self.__dbUserHost

	def getDbUserName(self):
		return self.__dbUserName

	def getDbUserPassword(self):
		return self.__dbUserPassword

	def sqlEscapeStr(self, mystr):
		return MySQLdb.escape_string(mystr)

	def sqlUnescapeStr(self, mystr):
		if type(mystr)!=type("string"):
			return mystr
		mystr=re.sub(r"""'(?P<sqlchar>['])""",r"""\g<sqlchar>""", mystr)	
		mystr=re.sub(r"\\r",r"\r", mystr)	
		mystr=re.sub(r"\\n",r"\n", mystr)	
		return mystr

	def createNewDb(self, dbName):
		if self.getDbState() == self.CONNECTED:
			raiseDbExcept("Already connected to db")
		self.dbName=dbName
		connection = MySQLdb.Connection(host=self.getDbUserHost(), user=self.getDbUserName(), passwd=self.getDbUserPassword())
		cursor=connection.cursor()
		cursor.execute("create database %s"%self.dbName)
		cursor.close()
		connection.select_db(self.dbName)
		self.__connection=connection
		self.setDbState(self.CONNECTED)


	def connectExistingDb(self, dbName):
		if self.getDbState() == self.CONNECTED:
			raiseDbExcept("Already connected to db")
		self.dbName=dbName
		connection = MySQLdb.Connection(host=self.getDbUserHost(), user=self.getDbUserName(), passwd=self.getDbUserPassword())
		connection.select_db(self.dbName)
		self.__connection=connection
		self.setDbState(self.CONNECTED)

	def dropExistingDb(self, dbName):
		connection = MySQLdb.Connection(host=self.getDbUserHost(), user=self.getDbUserName(), passwd=self.getDbUserPassword())
		connection.db.query("drop database %s"%dbName)
		connection.close()

	def openCursor(self, arraysize=10):
		cursor=self.__connection.cursor()
		#The arraysize determines how many results are fetched in cursor.execute()
		cursor.arraysize=arraysize
		return cursor

	def closeCursor(self, cursor):
		return cursor.close()

	def commitConnection(self):
		self.__connection.commit()

	def closeDb(self):
		self.commitConnection()
		self.__connection.close()
